/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import * as tslib_1 from "tslib";
import Url from 'url';
import classNames from 'classnames';
import React, { Component, Fragment } from 'react';
import * as Rx from 'rxjs';
import { 
// TODO: add type annotations
// @ts-ignore
EuiHeader, 
// @ts-ignore
EuiHeaderLogo, 
// @ts-ignore
EuiHeaderSection, 
// @ts-ignore
EuiHeaderSectionItem, 
// @ts-ignore
EuiHeaderSectionItemButton, EuiHorizontalRule, EuiIcon, EuiListGroup, 
// @ts-ignore
EuiListGroupItem, 
// @ts-ignore
EuiNavDrawer, 
// @ts-ignore
EuiNavDrawerFlyout, 
// @ts-ignore
EuiNavDrawerMenu, EuiOutsideClickDetector, 
// @ts-ignore
EuiShowFor, } from '@elastic/eui';
import { HeaderBreadcrumbs } from './header_breadcrumbs';
import { HeaderHelpMenu } from './header_help_menu';
import { HeaderNavControls } from './header_nav_controls';
import { injectI18n } from '@kbn/i18n/react';
import chrome from 'ui/chrome';
import { relativeToAbsolute } from 'ui/url/relative_to_absolute';
import { NavControlSide } from '../';
function extendRecentlyAccessedHistoryItem(navLinks, recentlyAccessed) {
    var href = relativeToAbsolute(chrome.addBasePath(recentlyAccessed.link));
    var navLink = navLinks.find(function (nl) { return href.startsWith(nl.subUrlBase); });
    return tslib_1.__assign({}, recentlyAccessed, { href: href, euiIconType: navLink ? navLink.euiIconType : undefined });
}
function extendNavLink(navLink) {
    return tslib_1.__assign({}, navLink, { href: navLink.lastSubUrl && !navLink.active ? navLink.lastSubUrl : navLink.url });
}
function findClosestAnchor(element) {
    var current = element;
    while (current) {
        if (current.tagName === 'A') {
            return current;
        }
        if (!current.parentElement || current.parentElement === document.body) {
            return undefined;
        }
        current = current.parentElement;
    }
}
var HeaderUI = /** @class */ (function (_super) {
    tslib_1.__extends(HeaderUI, _super);
    function HeaderUI(props) {
        var _this = _super.call(this, props) || this;
        _this.onNavClick = function (event) {
            var anchor = findClosestAnchor(event.nativeEvent.target);
            if (!anchor) {
                return;
            }
            var navLink = _this.state.navLinks.find(function (item) { return item.href === anchor.href; });
            if (navLink && navLink.disabled) {
                event.preventDefault();
                return;
            }
            if (!_this.state.forceNavigation ||
                event.isDefaultPrevented() ||
                event.altKey ||
                event.metaKey ||
                event.ctrlKey) {
                return;
            }
            var toParsed = Url.parse(anchor.href);
            var fromParsed = Url.parse(document.location.href);
            var sameProto = toParsed.protocol === fromParsed.protocol;
            var sameHost = toParsed.host === fromParsed.host;
            var samePath = toParsed.path === fromParsed.path;
            if (sameProto && sameHost && samePath) {
                if (toParsed.hash) {
                    document.location.reload();
                }
                // event.preventDefault() keeps the browser from seeing the new url as an update
                // and even setting window.location does not mimic that behavior, so instead
                // we use stopPropagation() to prevent angular from seeing the click and
                // starting a digest cycle/attempting to handle it in the router.
                event.stopPropagation();
            }
        };
        _this.toggleOpen = function () {
            _this.setState({
                mobileIsHidden: !_this.state.mobileIsHidden,
            });
            setTimeout(function () {
                _this.setState({
                    outsideClickDisabled: _this.state.mobileIsHidden ? true : false,
                });
            }, _this.getTimeoutMs(350));
        };
        _this.expandDrawer = function () {
            _this.timeoutExpand = setTimeout(function () {
                _this.setState({
                    isCollapsed: false,
                });
            }, _this.getTimeoutMs(750));
            _this.timeoutScrollbar = setTimeout(function () {
                _this.setState({
                    showScrollbar: true,
                });
            }, _this.getTimeoutMs(1200));
            // This prevents the drawer from collapsing when tabbing through children
            // by clearing the timeout thus cancelling the onBlur event (see focusOut).
            // This means isManagingFocus remains true as long as a child element
            // has focus. This is the case since React bubbles up onFocus and onBlur
            // events from the child elements.
            if (_this.timeoutID) {
                clearTimeout(_this.timeoutID);
            }
            if (!_this.state.isManagingFocus) {
                _this.setState({
                    isManagingFocus: true,
                });
            }
        };
        _this.collapseDrawer = function () {
            // Stop the expand animation
            if (_this.timeoutExpand) {
                clearTimeout(_this.timeoutExpand);
            }
            if (_this.timeoutScrollbar) {
                clearTimeout(_this.timeoutScrollbar);
            }
            _this.setState({
                flyoutIsAnimating: false,
                isCollapsed: true,
                flyoutIsCollapsed: true,
                mobileIsHidden: true,
                showScrollbar: false,
                outsideClickDisabled: true,
            });
            // Scrolls the menu and flyout back to top when the nav drawer collapses
            var menuEl = document.getElementById('navDrawerMenu');
            if (menuEl) {
                menuEl.scrollTop = 0;
            }
            var flyoutEl = document.getElementById('navDrawerFlyout');
            if (flyoutEl) {
                flyoutEl.scrollTop = 0;
            }
        };
        _this.focusOut = function () {
            // This collapses the drawer when no children have focus (i.e. tabbed out).
            // In other words, if focus does not bubble up from a child element, then
            // the drawer will collapse. See the corresponding block in expandDrawer
            // (called by onFocus) which cancels this operation via clearTimeout.
            _this.timeoutID = setTimeout(function () {
                if (_this.state.isManagingFocus) {
                    _this.setState({
                        isManagingFocus: false,
                    });
                    _this.collapseDrawer();
                }
            }, 0);
        };
        _this.expandFlyout = function () {
            _this.setState(function () { return ({
                flyoutIsCollapsed: !_this.state.flyoutIsCollapsed,
            }); });
            _this.setState({
                flyoutIsAnimating: true,
            });
        };
        _this.collapseFlyout = function () {
            _this.setState({ flyoutIsAnimating: true });
            setTimeout(function () {
                _this.setState({
                    flyoutIsCollapsed: true,
                });
            }, _this.getTimeoutMs(250));
        };
        _this.getTimeoutMs = function (defaultTimeout) {
            var uiSettings = chrome.getUiSettingsClient();
            return uiSettings.get('accessibility:disableAnimations') ? 0 : defaultTimeout;
        };
        _this.state = {
            isCollapsed: true,
            flyoutIsCollapsed: true,
            flyoutIsAnimating: false,
            navFlyoutTitle: '',
            navFlyoutContent: [],
            mobileIsHidden: true,
            showScrollbar: false,
            outsideClickDisabled: true,
            isManagingFocus: false,
            navLinks: [],
            recentlyAccessed: [],
            forceNavigation: false,
        };
        return _this;
    }
    HeaderUI.prototype.componentDidMount = function () {
        var _this = this;
        this.subscription = Rx.combineLatest(this.props.navLinks$, this.props.recentlyAccessed$, this.props.forceAppSwitcherNavigation$).subscribe({
            next: function (_a) {
                var _b = tslib_1.__read(_a, 3), navLinks = _b[0], recentlyAccessed = _b[1], forceNavigation = _b[2];
                _this.setState({
                    forceNavigation: forceNavigation,
                    navLinks: navLinks.map(function (navLink) { return extendNavLink(navLink); }),
                    recentlyAccessed: recentlyAccessed.map(function (ra) {
                        return extendRecentlyAccessedHistoryItem(navLinks, ra);
                    }),
                });
            },
        });
    };
    HeaderUI.prototype.componentWillUnmount = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    HeaderUI.prototype.renderLogo = function () {
        var _a = this.props, homeHref = _a.homeHref, intl = _a.intl;
        return (React.createElement(EuiHeaderLogo, { "data-test-subj": "logo", iconType: "logoKibana", onClick: this.onNavClick, href: homeHref, "aria-label": intl.formatMessage({
                id: 'common.ui.chrome.headerGlobalNav.goHomePageIconAriaLabel',
                defaultMessage: 'Go to home page',
            }) }));
    };
    HeaderUI.prototype.renderMenuTrigger = function () {
        return (React.createElement(EuiHeaderSectionItemButton, { "aria-label": "Toggle side navigation", onClick: this.toggleOpen },
            React.createElement(EuiIcon, { type: "apps", size: "m" })));
    };
    HeaderUI.prototype.render = function () {
        var _this = this;
        var _a = this.props, appTitle = _a.appTitle, breadcrumbs$ = _a.breadcrumbs$, isVisible = _a.isVisible, navControls = _a.navControls, helpExtension$ = _a.helpExtension$;
        var _b = this.state, navLinks = _b.navLinks, recentlyAccessed = _b.recentlyAccessed;
        if (!isVisible) {
            return null;
        }
        var leftNavControls = navControls.bySide[NavControlSide.Left];
        var rightNavControls = navControls.bySide[NavControlSide.Right];
        return (React.createElement(Fragment, null,
            React.createElement(EuiHeader, null,
                React.createElement(EuiHeaderSection, { grow: false },
                    React.createElement(EuiShowFor, { sizes: ['xs', 's'] },
                        React.createElement(EuiHeaderSectionItem, { border: "right" }, this.renderMenuTrigger())),
                    React.createElement(EuiHeaderSectionItem, { border: "right" }, this.renderLogo()),
                    React.createElement(HeaderNavControls, { navControls: leftNavControls })),
                React.createElement(HeaderBreadcrumbs, { appTitle: appTitle, "breadcrumbs$": breadcrumbs$ }),
                React.createElement(EuiHeaderSection, { side: "right" },
                    React.createElement(EuiHeaderSectionItem, null,
                        React.createElement(HeaderHelpMenu, { "helpExtension$": helpExtension$ })),
                    React.createElement(HeaderNavControls, { navControls: rightNavControls }))),
            React.createElement(EuiOutsideClickDetector, { onOutsideClick: function () { return _this.collapseDrawer(); }, isDisabled: this.state.outsideClickDisabled },
                React.createElement(EuiNavDrawer, { isCollapsed: this.state.isCollapsed, flyoutIsCollapsed: this.state.flyoutIsCollapsed, flyoutIsAnimating: this.state.flyoutIsAnimating, onMouseEnter: this.expandDrawer, onFocus: this.expandDrawer, onBlur: this.focusOut, onMouseLeave: this.collapseDrawer, mobileIsHidden: this.state.mobileIsHidden, showScrollbar: this.state.showScrollbar, "data-test-subj": classNames('navDrawer', this.state.flyoutIsAnimating
                        ? null
                        : this.state.isCollapsed
                            ? 'collapsed'
                            : 'expanded') },
                    React.createElement(EuiNavDrawerMenu, { id: "navDrawerMenu", onClick: this.onNavClick },
                        React.createElement(EuiListGroup, null,
                            React.createElement(EuiListGroupItem, { label: "Recently viewed", iconType: "clock", size: "s", style: { color: 'inherit' }, "aria-label": "Recently viewed items", onClick: function () { return _this.expandFlyout(); }, isDisabled: recentlyAccessed.length > 0 ? false : true, extraAction: {
                                    color: 'subdued',
                                    iconType: 'arrowRight',
                                    iconSize: 's',
                                    'aria-label': 'Expand to view recent apps and objects',
                                    onClick: function () { return _this.expandFlyout(); },
                                    alwaysShow: true,
                                } })),
                        React.createElement(EuiHorizontalRule, { margin: "none" }),
                        React.createElement(EuiListGroup, { "data-test-subj": "appsMenu" }, navLinks.map(function (navLink) {
                            return navLink.hidden ? null : (React.createElement(EuiListGroupItem, { key: navLink.id, label: navLink.title, href: navLink.href, iconType: navLink.euiIconType, size: "s", style: { color: 'inherit' }, "aria-label": navLink.title, isActive: navLink.active, "data-test-subj": "appLink" }));
                        }))),
                    React.createElement(EuiNavDrawerFlyout, { id: "navDrawerFlyout", title: "Recent items", isCollapsed: this.state.flyoutIsCollapsed, listItems: recentlyAccessed.map(function (item) { return ({
                            label: item.label,
                            href: item.href,
                            iconType: item.euiIconType,
                            size: 's',
                            style: { color: 'inherit' },
                            'aria-label': item.label,
                        }); }), onMouseLeave: this.collapseFlyout, wrapText: true })))));
    };
    return HeaderUI;
}(Component));
export var Header = injectI18n(HeaderUI);
