/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import * as tslib_1 from "tslib";
// @ts-ignore
import { setBounds } from 'ui/agg_types/buckets/date_histogram';
var vislibCharts = [
    'area',
    'gauge',
    'goal',
    'heatmap',
    'histogram',
    'horizontal_bar',
    'line',
];
export var getSchemas = function (vis, timeRange) {
    var createFormat = function (agg) {
        var format = agg.params.field ? agg.params.field.format.toJSON() : {};
        var formats = {
            date_range: function () { return ({ id: 'string' }); },
            percentile_ranks: function () { return ({ id: 'percent' }); },
            count: function () { return ({ id: 'number' }); },
            cardinality: function () { return ({ id: 'number' }); },
            date_histogram: function () { return ({
                id: 'date',
                params: {
                    pattern: agg.buckets.getScaledDateFormat(),
                },
            }); },
            terms: function () { return ({
                id: 'terms',
                params: tslib_1.__assign({ id: format.id, otherBucketLabel: agg.params.otherBucketLabel, missingBucketLabel: agg.params.missingBucketLabel }, format.params),
            }); },
            range: function () { return ({
                id: 'range',
                params: tslib_1.__assign({ id: format.id }, format.params),
            }); },
        };
        return formats[agg.type.name] ? formats[agg.type.name]() : format;
    };
    var createSchemaConfig = function (accessor, agg) {
        var schema = {
            accessor: accessor,
            format: {},
            params: {},
            aggType: agg.type.name,
        };
        if (agg.type.name === 'date_histogram') {
            agg.params.timeRange = timeRange;
            setBounds(agg, true);
        }
        if (agg.type.name === 'geohash_grid') {
            schema.params = {
                precision: agg.params.precision,
                useGeocentroid: agg.params.useGeocentroid,
            };
        }
        if ([
            'derivative',
            'moving_avg',
            'serial_diff',
            'cumulative_sum',
            'sum_bucket',
            'avg_bucket',
            'min_bucket',
            'max_bucket',
        ].includes(agg.type.name)) {
            var subAgg = agg.params.customMetric || agg.aggConfigs.byId[agg.params.metricAgg];
            schema.format = createFormat(subAgg);
        }
        else {
            schema.format = createFormat(agg);
        }
        return schema;
    };
    var cnt = 0;
    var schemas = {
        metric: [],
    };
    var responseAggs = vis.aggs.getResponseAggs().filter(function (agg) { return agg.enabled; });
    var isHierarchical = vis.isHierarchical();
    var metrics = responseAggs.filter(function (agg) { return agg.type.type === 'metrics'; });
    responseAggs.forEach(function (agg) {
        if (!agg.enabled) {
            cnt++;
            return;
        }
        var skipMetrics = false;
        var schemaName = agg.schema ? agg.schema.name || agg.schema : null;
        if (typeof schemaName === 'object') {
            schemaName = null;
        }
        if (!schemaName) {
            if (agg.type.name === 'geo_centroid') {
                schemaName = 'geo_centroid';
            }
            else {
                cnt++;
                return;
            }
        }
        if (schemaName === 'split') {
            schemaName = "split_" + (agg.params.row ? 'row' : 'column');
            skipMetrics = true;
        }
        if (!schemas[schemaName]) {
            schemas[schemaName] = [];
        }
        if (!isHierarchical || agg.type.type !== 'metrics') {
            schemas[schemaName].push(createSchemaConfig(cnt++, agg));
        }
        if (isHierarchical && (agg.type.type !== 'metrics' || metrics.length === responseAggs.length)) {
            metrics.forEach(function (metric) {
                var schemaConfig = createSchemaConfig(cnt++, metric);
                if (!skipMetrics) {
                    schemas.metric.push(schemaConfig);
                }
            });
        }
    });
    return schemas;
};
export var prepareJson = function (variable, data) {
    return variable + "='" + JSON.stringify(data)
        .replace(/\\/g, "\\\\")
        .replace(/'/g, "\\'") + "' ";
};
export var prepareString = function (variable, data) {
    return variable + "='" + data.replace(/\\/g, "\\\\").replace(/'/g, "\\'") + "' ";
};
export var buildPipelineVisFunction = {
    vega: function (visState) {
        return "vega " + prepareString('spec', visState.params.spec);
    },
    input_control_vis: function (visState) {
        return "input_control_vis " + prepareJson('visConfig', visState.params);
    },
    metrics: function (visState) {
        return "tsvb " + prepareJson('params', visState.params);
    },
    timelion: function (visState) {
        var expression = prepareString('expression', visState.params.expression);
        var interval = prepareString('interval', visState.params.interval);
        return "timelion_vis " + expression + interval;
    },
    markdown: function (visState) {
        var expression = prepareString('expression', visState.params.markdown);
        var visConfig = prepareJson('visConfig', visState.params);
        return "kibana_markdown " + expression + visConfig;
    },
    table: function (visState, schemas) {
        var visConfig = buildVisConfig.table(visState, schemas);
        return "kibana_table " + prepareJson('visConfig', visConfig);
    },
    metric: function (visState, schemas) {
        var visConfig = buildVisConfig.metric(visState, schemas);
        return "kibana_metric " + prepareJson('visConfig', visConfig);
    },
    tagcloud: function (visState, schemas) {
        var visConfig = buildVisConfig.tagcloud(visState, schemas);
        return "tagcloud " + prepareJson('visConfig', visConfig);
    },
    region_map: function (visState, schemas) {
        var visConfig = buildVisConfig.region_map(visState, schemas);
        return "regionmap " + prepareJson('visConfig', visConfig);
    },
    tile_map: function (visState, schemas) {
        var visConfig = buildVisConfig.tile_map(visState, schemas);
        return "tilemap " + prepareJson('visConfig', visConfig);
    },
    pie: function (visState, schemas) {
        var visConfig = buildVisConfig.pie(visState, schemas);
        return "kibana_pie " + prepareJson('visConfig', visConfig);
    },
};
var buildVisConfig = {
    table: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.dimensions = {
            metrics: schemas.metric,
            buckets: schemas.bucket || [],
            splitRow: schemas.split_row,
            splitColumn: schemas.split_column,
        };
        return visConfig;
    },
    metric: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.metric.metrics = schemas.metric;
        if (schemas.group) {
            visConfig.metric.bucket = schemas.group[0];
        }
        return visConfig;
    },
    tagcloud: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.metric = schemas.metric[0];
        if (schemas.segment) {
            visConfig.bucket = schemas.segment[0];
        }
        return visConfig;
    },
    region_map: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.metric = schemas.metric[0];
        if (schemas.segment) {
            visConfig.bucket = schemas.segment[0];
        }
        return visConfig;
    },
    tile_map: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.dimensions = {
            metric: schemas.metric[0],
            geohash: schemas.segment ? schemas.segment[0] : null,
            geocentroid: schemas.geo_centroid ? schemas.geo_centroid[0] : null,
        };
        return visConfig;
    },
    pie: function (visState, schemas) {
        var visConfig = visState.params;
        visConfig.dimensions = {
            metric: schemas.metric[0],
            buckets: schemas.segment,
            splitRow: schemas.split_row,
            splitColumn: schemas.split_column,
        };
        return visConfig;
    },
};
export var buildVislibDimensions = function (vis, timeRange) {
    var schemas = getSchemas(vis, timeRange);
    var dimensions = {
        x: schemas.segment ? schemas.segment[0] : null,
        y: schemas.metric,
        z: schemas.radius,
        width: schemas.width,
        series: schemas.group,
        splitRow: schemas.split_row,
        splitColumn: schemas.split_column,
    };
    if (schemas.segment) {
        var xAgg = vis.aggs.getResponseAggs()[dimensions.x.accessor];
        if (xAgg.type.name === 'date_histogram') {
            dimensions.x.params.date = true;
            dimensions.x.params.interval = xAgg.buckets.getInterval().asMilliseconds();
            dimensions.x.params.format = xAgg.buckets.getScaledDateFormat();
            dimensions.x.params.bounds = xAgg.buckets.getBounds();
        }
    }
    return dimensions;
};
// If not using the expression pipeline (i.e. visualize_data_loader), we need a mechanism to
// take a Vis object and decorate it with the necessary params (dimensions, bucket, metric, etc)
export var decorateVisObject = function (vis, params) {
    var schemas = getSchemas(vis, params.timeRange);
    var visConfig = vis.params;
    if (buildVisConfig[vis.type.name]) {
        visConfig = buildVisConfig[vis.type.name](vis, schemas);
        vis.params = visConfig;
    }
    else if (vislibCharts.includes(vis.type.name)) {
        visConfig.dimensions = buildVislibDimensions(vis, params.timeRange);
    }
};
export var buildPipeline = function (vis, params) {
    var searchSource = params.searchSource;
    var indexPattern = vis.indexPattern;
    var query = searchSource.getField('query');
    var filters = searchSource.getField('filter');
    var visState = vis.getCurrentState();
    // context
    var pipeline = "kibana | kibana_context ";
    if (query) {
        pipeline += prepareJson('query', query);
    }
    if (filters) {
        pipeline += prepareJson('filters', filters);
    }
    if (vis.savedSearchId) {
        pipeline += prepareString('savedSearchId', vis.savedSearchId);
    }
    pipeline += '| ';
    // request handler
    if (vis.type.requestHandler === 'courier') {
        pipeline += "esaggs\n    " + prepareString('index', indexPattern.id) + "\n    metricsAtAllLevels=" + vis.isHierarchical() + "\n    partialRows=" + (vis.params.showPartialRows || vis.type.requiresPartialRows || false) + "\n    " + prepareJson('aggConfigs', visState.aggs) + " | ";
    }
    var schemas = getSchemas(vis, params.timeRange);
    if (buildPipelineVisFunction[vis.type.name]) {
        pipeline += buildPipelineVisFunction[vis.type.name](visState, schemas);
    }
    else if (vislibCharts.includes(vis.type.name)) {
        var visConfig = visState.params;
        visConfig.dimensions = buildVislibDimensions(vis, params.timeRange);
        pipeline += "vislib " + prepareJson('visConfig', visState.params);
    }
    else {
        pipeline += "visualization type='" + vis.type.name + "'\n    " + prepareJson('visConfig', visState.params) + "\n    metricsAtAllLevels=" + vis.isHierarchical() + "\n    partialRows=" + (vis.params.showPartialRows || vis.type.name === 'tile_map') + " ";
        if (indexPattern) {
            pipeline += "" + prepareString('index', indexPattern.id);
        }
    }
    return pipeline;
};
